@@ -244,8 +244,11 @@ class CMySQLCursor(MySQLCursorAbstract):
         if not operation:
             return None
 
-        if not self._cnx or self._cnx.is_closed():
-            raise ProgrammingError("Cursor is not connected", 2055)
+        try:
+            if not self._cnx or self._cnx.is_closed():
+                raise ProgrammingError
+        except (ProgrammingError, ReferenceError) as err:
+            raise ProgrammingError("Cursor is not connected", 2055) from err
         self._cnx.handle_unread_result()
 
         stmt = ""
@@ -348,12 +351,35 @@ class CMySQLCursor(MySQLCursorAbstract):
             raise InterfaceError(f"Failed executing the operation; {err}") from None
 
     def executemany(self, operation, seq_params):
-        """Execute the given operation multiple times"""
+        """Execute the given operation multiple times
+
+        The executemany() method will execute the operation iterating
+        over the list of parameters in seq_params.
+
+        Example: Inserting 3 new employees and their phone number
+
+        data = [
+            ('Jane','555-001'),
+            ('Joe', '555-001'),
+            ('John', '555-003')
+            ]
+        stmt = "INSERT INTO employees (name, phone) VALUES ('%s','%s)"
+        cursor.executemany(stmt, data)
+
+        INSERT statements are optimized by batching the data, that is
+        using the MySQL multiple rows syntax.
+
+        Results are discarded! If they are needed, consider looping over
+        data using the execute() method.
+        """
         if not operation or not seq_params:
             return None
 
-        if not self._cnx:
-            raise ProgrammingError("Cursor is not connected")
+        try:
+            if not self._cnx:
+                raise ProgrammingError
+        except (ProgrammingError, ReferenceError) as err:
+            raise ProgrammingError("Cursor is not connected") from err
         self._cnx.handle_unread_result()
 
         if not isinstance(seq_params, (list, tuple)):
@@ -371,22 +397,18 @@ class CMySQLCursor(MySQLCursorAbstract):
 
         rowcnt = 0
         try:
+            # When processing read ops (e.g., SELECT), rowcnt is updated
+            # based on self._rowcount. For write ops (e.g., INSERT) is
+            # updated based on self._affected_rows.
+            # The variable self._description is None for write ops, that's
+            # why we use it as indicator for updating rowcnt.
             for params in seq_params:
                 self.execute(operation, params)
-                try:
-                    while True:
-                        if self._description:
-                            rowcnt += len(self._cnx.get_rows()[0])
-                        else:
-                            rowcnt += self._affected_rows
-                        if not self.nextset():
-                            break
-                except StopIteration:
-                    # No more results
-                    pass
-
+                if self.with_rows and self._cnx.unread_result:
+                    self.fetchall()
+                rowcnt += self._rowcount if self.description else self._affected_rows
         except (ValueError, TypeError) as err:
-            raise ProgrammingError(f"Failed executing the operation; {err}") from err
+            raise InterfaceError(f"Failed executing the operation; {err}") from None
 
         self._rowcount = rowcnt
         return None
@@ -430,9 +452,14 @@ class CMySQLCursor(MySQLCursorAbstract):
         try:
             argnames = []
             argtypes = []
+
+            # MySQL itself does support calling procedures with their full
+            # name <database>.<procedure_name>. It's necessary to split
+            # by '.' and grab the procedure name from procname.
+            procname_abs = procname.split(".")[-1]
             if args:
                 for idx, arg in enumerate(args):
-                    argname = argfmt.format(name=procname, index=idx + 1)
+                    argname = argfmt.format(name=procname_abs, index=idx + 1)
                     argnames.append(argname)
                     if isinstance(arg, tuple):
                         argtypes.append(f" CAST({argname} AS {arg[1]})")
@@ -759,6 +786,17 @@ class CMySQLCursorBuffered(CMySQLCursor):
         self._check_executed()
         return self._fetch_row()
 
+    @property
+    def with_rows(self):
+        """Returns whether the cursor could have rows returned
+
+        This property returns True when rows are available,
+        which will need to be fetched.
+
+        Returns True or False.
+        """
+        return self._rows is not None
+
 
 class CMySQLCursorRaw(CMySQLCursor):
     """Cursor using C Extension return raw results"""
@@ -1003,8 +1041,11 @@ class CMySQLCursorPrepared(CMySQLCursor):
         if not operation:
             return
 
-        if not self._cnx or self._cnx.is_closed():
-            raise ProgrammingError("Cursor is not connected", 2055)
+        try:
+            if not self._cnx or self._cnx.is_closed():
+                raise ProgrammingError
+        except (ProgrammingError, ReferenceError) as err:
+            raise ProgrammingError("Cursor is not connected", 2055) from err
 
         self._cnx.handle_unread_result(prepared=True)
 
