@@ -1,4 +1,4 @@
-# Copyright (c) 2020, 2021, Oracle and/or its affiliates.
+# Copyright (c) 2020, 2022, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -40,7 +40,6 @@ Requires and comes with MySQL Connector/Python v8.0.22 and later:
 """
 
 import warnings
-import sys
 
 from datetime import datetime
 
@@ -48,19 +47,18 @@ from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
 from django.db import IntegrityError
 from django.db.backends.base.base import BaseDatabaseWrapper
-from django.db import utils
-from django.utils.functional import cached_property
 from django.utils import dateparse, timezone
+from django.utils.functional import cached_property
 
 try:
     import mysql.connector
+
     from mysql.connector.conversion import MySQLConverter
 except ImportError as err:
-    raise ImproperlyConfigured(
-        "Error loading mysql.connector module: {0}".format(err))
+    raise ImproperlyConfigured(f"Error loading mysql.connector module: {err}") from err
 
 try:
-    from _mysql_connector import datetime_to_mysql, time_to_mysql
+    from _mysql_connector import datetime_to_mysql
 except ImportError:
     HAVE_CEXT = False
 else:
@@ -68,12 +66,11 @@ else:
 
 from .client import DatabaseClient
 from .creation import DatabaseCreation
-from .introspection import DatabaseIntrospection
-from .validation import DatabaseValidation
 from .features import DatabaseFeatures
+from .introspection import DatabaseIntrospection
 from .operations import DatabaseOperations
 from .schema import DatabaseSchemaEditor
-
+from .validation import DatabaseValidation
 
 Error = mysql.connector.Error
 DatabaseError = mysql.connector.DatabaseError
@@ -83,19 +80,20 @@ ProgrammingError = mysql.connector.ProgrammingError
 
 
 def adapt_datetime_with_timezone_support(value):
-    # Equivalent to DateTimeField.get_db_prep_value. Used only by raw SQL.
+    """Equivalent to DateTimeField.get_db_prep_value. Used only by raw SQL."""
     if settings.USE_TZ:
         if timezone.is_naive(value):
-            warnings.warn("MySQL received a naive datetime (%s)"
-                          " while time zone support is active." % value,
-                          RuntimeWarning)
+            warnings.warn(
+                f"MySQL received a naive datetime ({value})"
+                " while time zone support is active.",
+                RuntimeWarning,
+            )
             default_timezone = timezone.get_default_timezone()
             value = timezone.make_aware(value, default_timezone)
         value = value.astimezone(timezone.utc).replace(tzinfo=None)
     if HAVE_CEXT:
         return datetime_to_mysql(value)
-    else:
-        return value.strftime("%Y-%m-%d %H:%M:%S.%f")
+    return value.strftime("%Y-%m-%d %H:%M:%S.%f")
 
 
 class CursorWrapper:
@@ -105,6 +103,7 @@ class CursorWrapper:
     Connector/Python. If buffered option is True in those options,
     MySQLCursorBuffered will be used.
     """
+
     codes_for_integrityerror = (
         1048,  # Column cannot be null
         1690,  # BIGINT UNSIGNED value is out of range
@@ -115,7 +114,8 @@ class CursorWrapper:
     def __init__(self, cursor):
         self.cursor = cursor
 
-    def _adapt_execute_args_dict(self, args):
+    @staticmethod
+    def _adapt_execute_args_dict(args):
         if not args:
             return args
         new_args = dict(args)
@@ -125,7 +125,8 @@ class CursorWrapper:
 
         return new_args
 
-    def _adapt_execute_args(self, args):
+    @staticmethod
+    def _adapt_execute_args(args):
         if not args:
             return args
         new_args = list(args)
@@ -147,9 +148,9 @@ class CursorWrapper:
             new_args = self._adapt_execute_args(args)
         try:
             return self.cursor.execute(query, new_args)
-        except mysql.connector.OperationalError as e:
-            if e.args[0] in self.codes_for_integrityerror:
-                raise IntegrityError(*tuple(e.args))
+        except mysql.connector.OperationalError as exc:
+            if exc.args[0] in self.codes_for_integrityerror:
+                raise IntegrityError(*tuple(exc.args)) from None
             raise
 
     def executemany(self, query, args):
@@ -160,55 +161,57 @@ class CursorWrapper:
         """
         try:
             return self.cursor.executemany(query, args)
-        except mysql.connector.OperationalError as e:
-            if e.args[0] in self.codes_for_integrityerror:
-                raise IntegrityError(*tuple(e.args))
+        except mysql.connector.OperationalError as exc:
+            if exc.args[0] in self.codes_for_integrityerror:
+                raise IntegrityError(*tuple(exc.args)) from None
             raise
 
     def __getattr__(self, attr):
-        """Return attribute of wrapped cursor"""
+        """Return an attribute of wrapped cursor"""
         return getattr(self.cursor, attr)
 
     def __iter__(self):
-        """Returns iterator over wrapped cursor"""
+        """Return an iterator over wrapped cursor"""
         return iter(self.cursor)
 
 
 class DatabaseWrapper(BaseDatabaseWrapper):
-    vendor = 'mysql'
+    """Represent a database connection."""
+
+    vendor = "mysql"
     # This dictionary maps Field objects to their associated MySQL column
     # types, as strings. Column-type strings can contain format strings; they'll
     # be interpolated against the values of Field.__dict__ before being output.
     # If a column type is set to None, it won't be included in the output.
     data_types = {
-        'AutoField': 'integer AUTO_INCREMENT',
-        'BigAutoField': 'bigint AUTO_INCREMENT',
-        'BinaryField': 'longblob',
-        'BooleanField': 'bool',
-        'CharField': 'varchar(%(max_length)s)',
-        'DateField': 'date',
-        'DateTimeField': 'datetime(6)',
-        'DecimalField': 'numeric(%(max_digits)s, %(decimal_places)s)',
-        'DurationField': 'bigint',
-        'FileField': 'varchar(%(max_length)s)',
-        'FilePathField': 'varchar(%(max_length)s)',
-        'FloatField': 'double precision',
-        'IntegerField': 'integer',
-        'BigIntegerField': 'bigint',
-        'IPAddressField': 'char(15)',
-        'GenericIPAddressField': 'char(39)',
-        'JSONField': 'json',
-        'NullBooleanField': 'bool',
-        'OneToOneField': 'integer',
-        'PositiveBigIntegerField': 'bigint UNSIGNED',
-        'PositiveIntegerField': 'integer UNSIGNED',
-        'PositiveSmallIntegerField': 'smallint UNSIGNED',
-        'SlugField': 'varchar(%(max_length)s)',
-        'SmallAutoField': 'smallint AUTO_INCREMENT',
-        'SmallIntegerField': 'smallint',
-        'TextField': 'longtext',
-        'TimeField': 'time(6)',
-        'UUIDField': 'char(32)',
+        "AutoField": "integer AUTO_INCREMENT",
+        "BigAutoField": "bigint AUTO_INCREMENT",
+        "BinaryField": "longblob",
+        "BooleanField": "bool",
+        "CharField": "varchar(%(max_length)s)",
+        "DateField": "date",
+        "DateTimeField": "datetime(6)",
+        "DecimalField": "numeric(%(max_digits)s, %(decimal_places)s)",
+        "DurationField": "bigint",
+        "FileField": "varchar(%(max_length)s)",
+        "FilePathField": "varchar(%(max_length)s)",
+        "FloatField": "double precision",
+        "IntegerField": "integer",
+        "BigIntegerField": "bigint",
+        "IPAddressField": "char(15)",
+        "GenericIPAddressField": "char(39)",
+        "JSONField": "json",
+        "NullBooleanField": "bool",
+        "OneToOneField": "integer",
+        "PositiveBigIntegerField": "bigint UNSIGNED",
+        "PositiveIntegerField": "integer UNSIGNED",
+        "PositiveSmallIntegerField": "smallint UNSIGNED",
+        "SlugField": "varchar(%(max_length)s)",
+        "SmallAutoField": "smallint AUTO_INCREMENT",
+        "SmallIntegerField": "smallint",
+        "TextField": "longtext",
+        "TimeField": "time(6)",
+        "UUIDField": "char(32)",
     }
 
     # For these data types:
@@ -217,25 +220,32 @@ class DatabaseWrapper(BaseDatabaseWrapper):
     # - all versions of MySQL doesn't support full width database
     #   indexes
     _limited_data_types = (
-        'tinyblob', 'blob', 'mediumblob', 'longblob', 'tinytext', 'text',
-        'mediumtext', 'longtext', 'json',
+        "tinyblob",
+        "blob",
+        "mediumblob",
+        "longblob",
+        "tinytext",
+        "text",
+        "mediumtext",
+        "longtext",
+        "json",
     )
 
     operators = {
-        'exact': '= %s',
-        'iexact': 'LIKE %s',
-        'contains': 'LIKE BINARY %s',
-        'icontains': 'LIKE %s',
-        'regex': 'REGEXP BINARY %s',
-        'iregex': 'REGEXP %s',
-        'gt': '> %s',
-        'gte': '>= %s',
-        'lt': '< %s',
-        'lte': '<= %s',
-        'startswith': 'LIKE BINARY %s',
-        'endswith': 'LIKE BINARY %s',
-        'istartswith': 'LIKE %s',
-        'iendswith': 'LIKE %s',
+        "exact": "= %s",
+        "iexact": "LIKE %s",
+        "contains": "LIKE BINARY %s",
+        "icontains": "LIKE %s",
+        "regex": "REGEXP BINARY %s",
+        "iregex": "REGEXP %s",
+        "gt": "> %s",
+        "gte": ">= %s",
+        "lt": "< %s",
+        "lte": "<= %s",
+        "startswith": "LIKE BINARY %s",
+        "endswith": "LIKE BINARY %s",
+        "istartswith": "LIKE %s",
+        "iendswith": "LIKE %s",
     }
 
     # The patterns below are used to generate SQL pattern lookup clauses when
@@ -248,19 +258,20 @@ class DatabaseWrapper(BaseDatabaseWrapper):
     # the LIKE operator.
     pattern_esc = r"REPLACE(REPLACE(REPLACE({}, '\\', '\\\\'), '%%', '\%%'), '_', '\_')"
     pattern_ops = {
-        'contains': "LIKE BINARY CONCAT('%%', {}, '%%')",
-        'icontains': "LIKE CONCAT('%%', {}, '%%')",
-        'startswith': "LIKE BINARY CONCAT({}, '%%')",
-        'istartswith': "LIKE CONCAT({}, '%%')",
-        'endswith': "LIKE BINARY CONCAT('%%', {})",
-        'iendswith': "LIKE CONCAT('%%', {})",
+        "contains": "LIKE BINARY CONCAT('%%', {}, '%%')",
+        "icontains": "LIKE CONCAT('%%', {}, '%%')",
+        "startswith": "LIKE BINARY CONCAT({}, '%%')",
+        "istartswith": "LIKE CONCAT({}, '%%')",
+        "endswith": "LIKE BINARY CONCAT('%%', {})",
+        "iendswith": "LIKE CONCAT('%%', {})",
     }
 
+    isolation_level = None
     isolation_levels = {
-        'read uncommitted',
-        'read committed',
-        'repeatable read',
-        'serializable',
+        "read uncommitted",
+        "read committed",
+        "repeatable read",
+        "serializable",
     }
 
     Database = mysql.connector
@@ -274,24 +285,24 @@ class DatabaseWrapper(BaseDatabaseWrapper):
     validation_class = DatabaseValidation
 
     def __init__(self, *args, **kwargs):
-        super(DatabaseWrapper, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
-        options = self.settings_dict.get('OPTIONS')
+        options = self.settings_dict.get("OPTIONS")
         if options:
-            self._use_pure = options.get('use_pure', not HAVE_CEXT)
+            self._use_pure = options.get("use_pure", not HAVE_CEXT)
             converter_class = options.get(
-                'converter_class',
+                "converter_class",
                 DjangoMySQLConverter,
             )
             if not issubclass(converter_class, DjangoMySQLConverter):
                 raise ProgrammingError(
-                    'Converter class should be a subclass of '
-                    'mysql.connector.django.base.DjangoMySQLConverter'
+                    "Converter class should be a subclass of "
+                    "mysql.connector.django.base.DjangoMySQLConverter"
                 )
             self.converter = converter_class()
         else:
-             self.converter = DjangoMySQLConverter()
-             self._use_pure = not HAVE_CEXT
+            self.converter = DjangoMySQLConverter()
+            self._use_pure = not HAVE_CEXT
 
     def __getattr__(self, attr):
         if attr.startswith("mysql_is"):
@@ -300,45 +311,58 @@ class DatabaseWrapper(BaseDatabaseWrapper):
 
     def get_connection_params(self):
         kwargs = {
-            'charset': 'utf8',
-            'use_unicode': True,
-            'buffered': False,
-            'consume_results': True,
+            "charset": "utf8",
+            "use_unicode": True,
+            "buffered": False,
+            "consume_results": True,
         }
 
         settings_dict = self.settings_dict
 
-        if settings_dict['USER']:
-            kwargs['user'] = settings_dict['USER']
-        if settings_dict['NAME']:
-            kwargs['database'] = settings_dict['NAME']
-        if settings_dict['PASSWORD']:
-            kwargs['passwd'] = settings_dict['PASSWORD']
-        if settings_dict['HOST'].startswith('/'):
-            kwargs['unix_socket'] = settings_dict['HOST']
-        elif settings_dict['HOST']:
-            kwargs['host'] = settings_dict['HOST']
-        if settings_dict['PORT']:
-            kwargs['port'] = int(settings_dict['PORT'])
+        if settings_dict["USER"]:
+            kwargs["user"] = settings_dict["USER"]
+        if settings_dict["NAME"]:
+            kwargs["database"] = settings_dict["NAME"]
+        if settings_dict["PASSWORD"]:
+            kwargs["passwd"] = settings_dict["PASSWORD"]
+        if settings_dict["HOST"].startswith("/"):
+            kwargs["unix_socket"] = settings_dict["HOST"]
+        elif settings_dict["HOST"]:
+            kwargs["host"] = settings_dict["HOST"]
+        if settings_dict["PORT"]:
+            kwargs["port"] = int(settings_dict["PORT"])
 
         # Raise exceptions for database warnings if DEBUG is on
-        kwargs['raise_on_warnings'] = settings.DEBUG
+        kwargs["raise_on_warnings"] = settings.DEBUG
 
-        kwargs['client_flags'] = [
+        kwargs["client_flags"] = [
             # Need potentially affected rows on UPDATE
             mysql.connector.constants.ClientFlag.FOUND_ROWS,
         ]
+
         try:
-            kwargs.update(settings_dict['OPTIONS'])
+            options = settings_dict["OPTIONS"].copy()
+            isolation_level = options.pop("isolation_level")
+            if isolation_level:
+                isolation_level = isolation_level.lower()
+                if isolation_level not in self.isolation_levels:
+                    valid_levels = ", ".join(
+                        f"'{level}'" for level in sorted(self.isolation_levels)
+                    )
+                    raise ImproperlyConfigured(
+                        f"Invalid transaction isolation level '{isolation_level}' "
+                        f"specified.\nUse one of {valid_levels}, or None."
+                    )
+            self.isolation_level = isolation_level
+            kwargs.update(options)
         except KeyError:
             # OPTIONS missing is OK
             pass
-
         return kwargs
 
     def get_new_connection(self, conn_params):
-        if not 'converter_class' in conn_params:
-            conn_params['converter_class'] = DjangoMySQLConverter
+        if "converter_class" not in conn_params:
+            conn_params["converter_class"] = DjangoMySQLConverter
         cnx = mysql.connector.connect(**conn_params)
 
         return cnx
@@ -350,17 +374,23 @@ class DatabaseWrapper(BaseDatabaseWrapper):
             # a recently inserted row will return when the field is tested
             # for NULL. Disabling this brings this aspect of MySQL in line
             # with SQL standards.
-            assignments.append('SET SQL_AUTO_IS_NULL = 0')
+            assignments.append("SET SQL_AUTO_IS_NULL = 0")
+
+        if self.isolation_level:
+            assignments.append(
+                "SET SESSION TRANSACTION ISOLATION LEVEL "
+                f"{self.isolation_level.upper()}"
+            )
 
         if assignments:
             with self.cursor() as cursor:
-                cursor.execute('; '.join(assignments))
+                cursor.execute("; ".join(assignments))
 
-        if 'AUTOCOMMIT' in self.settings_dict:
+        if "AUTOCOMMIT" in self.settings_dict:
             try:
-                self.set_autocommit(self.settings_dict['AUTOCOMMIT'])
+                self.set_autocommit(self.settings_dict["AUTOCOMMIT"])
             except AttributeError:
-                self._set_autocommit(self.settings_dict['AUTOCOMMIT'])
+                self._set_autocommit(self.settings_dict["AUTOCOMMIT"])
 
     def create_cursor(self, name=None):
         cursor = self.connection.cursor()
@@ -383,7 +413,7 @@ class DatabaseWrapper(BaseDatabaseWrapper):
         need to be re-enabled.
         """
         with self.cursor() as cursor:
-            cursor.execute('SET foreign_key_checks=0')
+            cursor.execute("SET foreign_key_checks=0")
         return True
 
     def enable_constraint_checking(self):
@@ -395,7 +425,7 @@ class DatabaseWrapper(BaseDatabaseWrapper):
         self.needs_rollback, needs_rollback = False, self.needs_rollback
         try:
             with self.cursor() as cursor:
-                cursor.execute('SET foreign_key_checks=1')
+                cursor.execute("SET foreign_key_checks=1")
         finally:
             self.needs_rollback = needs_rollback
 
@@ -411,32 +441,40 @@ class DatabaseWrapper(BaseDatabaseWrapper):
             if table_names is None:
                 table_names = self.introspection.table_names(cursor)
             for table_name in table_names:
-                primary_key_column_name = self.introspection.get_primary_key_column(cursor, table_name)
+                primary_key_column_name = self.introspection.get_primary_key_column(
+                    cursor, table_name
+                )
                 if not primary_key_column_name:
                     continue
                 key_columns = self.introspection.get_key_columns(cursor, table_name)
-                for column_name, referenced_table_name, referenced_column_name in key_columns:
+                for (
+                    column_name,
+                    referenced_table_name,
+                    referenced_column_name,
+                ) in key_columns:
                     cursor.execute(
-                        """
-                        SELECT REFERRING.`%s`, REFERRING.`%s` FROM `%s` as REFERRING
-                        LEFT JOIN `%s` as REFERRED
-                        ON (REFERRING.`%s` = REFERRED.`%s`)
-                        WHERE REFERRING.`%s` IS NOT NULL AND REFERRED.`%s` IS NULL
-                        """ % (
-                            primary_key_column_name, column_name, table_name,
-                            referenced_table_name, column_name, referenced_column_name,
-                            column_name, referenced_column_name,
+                        f"""
+                        SELECT REFERRING.`{primary_key_column_name}`,
+                        REFERRING.`{column_name}`
+                        FROM `{table_name}` as REFERRING
+                        LEFT JOIN `{referenced_table_name}` as REFERRED
+                        ON (
+                            REFERRING.`{column_name}` =
+                            REFERRED.`{referenced_column_name}`
                         )
+                        WHERE REFERRING.`{column_name}` IS NOT NULL
+                        AND REFERRED.`{referenced_column_name}` IS NULL
+                        """
                     )
                     for bad_row in cursor.fetchall():
                         raise IntegrityError(
-                            "The row in table '%s' with primary key '%s' has an invalid "
-                            "foreign key: %s.%s contains a value '%s' that does not "
-                            "have a corresponding value in %s.%s."
-                            % (
-                                table_name, bad_row[0], table_name, column_name,
-                                bad_row[1], referenced_table_name, referenced_column_name,
-                            )
+                            f"The row in table '{table_name}' with primary "
+                            f"key '{bad_row[0]}' has an invalid foreign key: "
+                            f"{table_name}.{column_name} contains a value "
+                            f"'{bad_row[1]}' that does not have a "
+                            f"corresponding value in "
+                            f"{referenced_table_name}."
+                            f"{referenced_column_name}."
                         )
 
     def is_usable(self):
@@ -448,52 +486,60 @@ class DatabaseWrapper(BaseDatabaseWrapper):
             return True
 
     @cached_property
-    def display_name(self):
-        return 'MySQL'
+    @staticmethod
+    def display_name():
+        """Display name."""
+        return "MySQL"
 
     @cached_property
     def data_type_check_constraints(self):
+        """Mapping of Field objects to their SQL for CHECK constraints."""
         if self.features.supports_column_check_constraints:
             check_constraints = {
-                'PositiveBigIntegerField': '`%(column)s` >= 0',
-                'PositiveIntegerField': '`%(column)s` >= 0',
-                'PositiveSmallIntegerField': '`%(column)s` >= 0',
+                "PositiveBigIntegerField": "`%(column)s` >= 0",
+                "PositiveIntegerField": "`%(column)s` >= 0",
+                "PositiveSmallIntegerField": "`%(column)s` >= 0",
             }
             return check_constraints
         return {}
 
     @cached_property
     def mysql_server_data(self):
+        """Return MySQL server data."""
         with self.temporary_connection() as cursor:
             # Select some server variables and test if the time zone
             # definitions are installed. CONVERT_TZ returns NULL if 'UTC'
             # timezone isn't loaded into the mysql.time_zone table.
-            cursor.execute("""
+            cursor.execute(
+                """
                 SELECT VERSION(),
                        @@sql_mode,
                        @@default_storage_engine,
                        @@sql_auto_is_null,
                        @@lower_case_table_names,
                        CONVERT_TZ('2001-01-01 01:00:00', 'UTC', 'UTC') IS NOT NULL
-            """)
+            """
+            )
             row = cursor.fetchone()
         return {
-            'version': row[0],
-            'sql_mode': row[1],
-            'default_storage_engine': row[2],
-            'sql_auto_is_null': bool(row[3]),
-            'lower_case_table_names': bool(row[4]),
-            'has_zoneinfo_database': bool(row[5]),
+            "version": row[0],
+            "sql_mode": row[1],
+            "default_storage_engine": row[2],
+            "sql_auto_is_null": bool(row[3]),
+            "lower_case_table_names": bool(row[4]),
+            "has_zoneinfo_database": bool(row[5]),
         }
 
     @cached_property
     def mysql_server_info(self):
+        """Return MySQL version."""
         with self.temporary_connection() as cursor:
-            cursor.execute('SELECT VERSION()')
+            cursor.execute("SELECT VERSION()")
             return cursor.fetchone()[0]
 
     @cached_property
     def mysql_version(self):
+        """Return MySQL version."""
         config = self.get_connection_params()
         with mysql.connector.connect(**config) as conn:
             server_version = conn.get_server_version()
@@ -501,26 +547,33 @@ class DatabaseWrapper(BaseDatabaseWrapper):
 
     @cached_property
     def sql_mode(self):
+        """Return SQL mode."""
         with self.cursor() as cursor:
-            cursor.execute('SELECT @@sql_mode')
+            cursor.execute("SELECT @@sql_mode")
             sql_mode = cursor.fetchone()
-        return set(sql_mode[0].split(',') if sql_mode else ())
+        return set(sql_mode[0].split(",") if sql_mode else ())
 
     @property
     def use_pure(self):
+        """Return True if pure Python version is being used."""
         return self._use_pure
 
 
 class DjangoMySQLConverter(MySQLConverter):
     """Custom converter for Django."""
-    def _TIME_to_python(self, value, dsc=None):
+
+    # pylint: disable=unused-argument
+
+    @staticmethod
+    def _time_to_python(value, dsc=None):
         """Return MySQL TIME data type as datetime.time()
 
         Returns datetime.time()
         """
-        return dateparse.parse_time(value.decode('utf-8'))
+        return dateparse.parse_time(value.decode("utf-8"))
 
-    def __DATETIME_to_python(self, value, dsc=None):
+    @staticmethod
+    def _datetime_to_python(value, dsc=None):
         """Connector/Python always returns naive datetime.datetime
 
         Connector/Python always returns naive timestamps since MySQL has
@@ -531,13 +584,16 @@ class DjangoMySQLConverter(MySQLConverter):
         """
         if not value:
             return None
-        dt = MySQLConverter._DATETIME_to_python(self, value)
+
+        dt = MySQLConverter._datetime_to_python(value)
         if dt is None:
             return None
         if settings.USE_TZ and timezone.is_naive(dt):
             dt = dt.replace(tzinfo=timezone.utc)
         return dt
 
+    # pylint: enable=unused-argument
+
     def _safestring_to_mysql(self, value):
         return self._str_to_mysql(value)
 
