@@ -1,4 +1,4 @@
-# Copyright (c) 2020, 2021, Oracle and/or its affiliates.
+# Copyright (c) 2020, 2022, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -26,88 +26,95 @@
 # along with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 
+"""Database Introspection."""
+
 from collections import namedtuple
 
 import sqlparse
 
-from mysql.connector.constants import FieldType
-
 from django import VERSION as DJANGO_VERSION
 from django.db.backends.base.introspection import (
-    BaseDatabaseIntrospection, FieldInfo as BaseFieldInfo, TableInfo,
+    BaseDatabaseIntrospection,
+    FieldInfo as BaseFieldInfo,
+    TableInfo,
 )
 from django.db.models import Index
 from django.utils.datastructures import OrderedSet
 
+from mysql.connector.constants import FieldType
+
 FieldInfo = namedtuple(
-    'FieldInfo',
-    BaseFieldInfo._fields + ('extra', 'is_unsigned', 'has_json_constraint')
+    "FieldInfo",
+    BaseFieldInfo._fields + ("extra", "is_unsigned", "has_json_constraint"),
 )
 if DJANGO_VERSION < (3, 2, 0):
     InfoLine = namedtuple(
-        'InfoLine',
-        'col_name data_type max_len num_prec num_scale extra column_default '
-        'is_unsigned'
+        "InfoLine",
+        "col_name data_type max_len num_prec num_scale extra column_default "
+        "is_unsigned",
     )
 else:
     InfoLine = namedtuple(
-        'InfoLine',
-        'col_name data_type max_len num_prec num_scale extra column_default '
-        'collation is_unsigned'
+        "InfoLine",
+        "col_name data_type max_len num_prec num_scale extra column_default "
+        "collation is_unsigned",
     )
 
 
 class DatabaseIntrospection(BaseDatabaseIntrospection):
+    """Encapsulate backend-specific introspection utilities."""
 
     data_types_reverse = {
-        FieldType.BLOB: 'TextField',
-        FieldType.DECIMAL: 'DecimalField',
-        FieldType.NEWDECIMAL: 'DecimalField',
-        FieldType.DATE: 'DateField',
-        FieldType.DATETIME: 'DateTimeField',
-        FieldType.DOUBLE: 'FloatField',
-        FieldType.FLOAT: 'FloatField',
-        FieldType.INT24: 'IntegerField',
-        FieldType.LONG: 'IntegerField',
-        FieldType.LONGLONG: 'BigIntegerField',
-        FieldType.SHORT: 'SmallIntegerField',
-        FieldType.STRING: 'CharField',
-        FieldType.TIME: 'TimeField',
-        FieldType.TIMESTAMP: 'DateTimeField',
-        FieldType.TINY: 'IntegerField',
-        FieldType.TINY_BLOB: 'TextField',
-        FieldType.MEDIUM_BLOB: 'TextField',
-        FieldType.LONG_BLOB: 'TextField',
-        FieldType.VAR_STRING: 'CharField',
+        FieldType.BLOB: "TextField",
+        FieldType.DECIMAL: "DecimalField",
+        FieldType.NEWDECIMAL: "DecimalField",
+        FieldType.DATE: "DateField",
+        FieldType.DATETIME: "DateTimeField",
+        FieldType.DOUBLE: "FloatField",
+        FieldType.FLOAT: "FloatField",
+        FieldType.INT24: "IntegerField",
+        FieldType.LONG: "IntegerField",
+        FieldType.LONGLONG: "BigIntegerField",
+        FieldType.SHORT: "SmallIntegerField",
+        FieldType.STRING: "CharField",
+        FieldType.TIME: "TimeField",
+        FieldType.TIMESTAMP: "DateTimeField",
+        FieldType.TINY: "IntegerField",
+        FieldType.TINY_BLOB: "TextField",
+        FieldType.MEDIUM_BLOB: "TextField",
+        FieldType.LONG_BLOB: "TextField",
+        FieldType.VAR_STRING: "CharField",
     }
 
     def get_field_type(self, data_type, description):
         field_type = super().get_field_type(data_type, description)
-        if 'auto_increment' in description.extra:
-            if field_type == 'IntegerField':
-                return 'AutoField'
-            elif field_type == 'BigIntegerField':
-                return 'BigAutoField'
-            elif field_type == 'SmallIntegerField':
-                return 'SmallAutoField'
+        if "auto_increment" in description.extra:
+            if field_type == "IntegerField":
+                return "AutoField"
+            if field_type == "BigIntegerField":
+                return "BigAutoField"
+            if field_type == "SmallIntegerField":
+                return "SmallAutoField"
         if description.is_unsigned:
-            if field_type == 'BigIntegerField':
-                return 'PositiveBigIntegerField'
-            elif field_type == 'IntegerField':
-                return 'PositiveIntegerField'
-            elif field_type == 'SmallIntegerField':
-                return 'PositiveSmallIntegerField'
+            if field_type == "BigIntegerField":
+                return "PositiveBigIntegerField"
+            if field_type == "IntegerField":
+                return "PositiveIntegerField"
+            if field_type == "SmallIntegerField":
+                return "PositiveSmallIntegerField"
         # JSON data type is an alias for LONGTEXT in MariaDB, use check
         # constraints clauses to introspect JSONField.
         if description.has_json_constraint:
-            return 'JSONField'
+            return "JSONField"
         return field_type
 
     def get_table_list(self, cursor):
         """Return a list of table and view names in the current database."""
         cursor.execute("SHOW FULL TABLES")
-        return [TableInfo(row[0], {'BASE TABLE': 't', 'VIEW': 'v'}.get(row[1]))
-                for row in cursor.fetchall()]
+        return [
+            TableInfo(row[0], {"BASE TABLE": "t", "VIEW": "v"}.get(row[1]))
+            for row in cursor.fetchall()
+        ]
 
     def get_table_description(self, cursor, table_name):
         """
@@ -116,21 +123,25 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
         """
         json_constraints = {}
         # A default collation for the given table.
-        cursor.execute("""
+        cursor.execute(
+            """
             SELECT  table_collation
             FROM    information_schema.tables
             WHERE   table_schema = DATABASE()
             AND     table_name = %s
-        """, [table_name])
+        """,
+            [table_name],
+        )
         row = cursor.fetchone()
-        default_column_collation = row[0] if row else ''
+        default_column_collation = row[0] if row else ""
         # information_schema database gives more accurate results for some figures:
         # - varchar length returned by cursor.description is an internal length,
         #   not visible length (#5725)
         # - precision and scale (for decimal fields) (#5014)
         # - auto_increment is not available in cursor.description
         if DJANGO_VERSION < (3, 2, 0):
-            cursor.execute("""
+            cursor.execute(
+                """
                 SELECT
                     column_name, data_type, character_maximum_length,
                     numeric_precision, numeric_scale, extra, column_default,
@@ -140,9 +151,12 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
                     END AS is_unsigned
                 FROM information_schema.columns
                 WHERE table_name = %s AND table_schema = DATABASE()
-            """, [table_name])
+            """,
+                [table_name],
+            )
         else:
-            cursor.execute("""
+            cursor.execute(
+                """
                 SELECT
                     column_name, data_type, character_maximum_length,
                     numeric_precision, numeric_scale, extra, column_default,
@@ -156,10 +170,14 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
                     END AS is_unsigned
                 FROM information_schema.columns
                 WHERE table_name = %s AND table_schema = DATABASE()
-            """, [default_column_collation, table_name])
+            """,
+                [default_column_collation, table_name],
+            )
         field_info = {line[0]: InfoLine(*line) for line in cursor.fetchall()}
 
-        cursor.execute("SELECT * FROM %s LIMIT 1" % self.connection.ops.quote_name(table_name))
+        cursor.execute(
+            f"SELECT * FROM {self.connection.ops.quote_name(table_name)} LIMIT 1"
+        )
 
         def to_int(i):
             return int(i) if i is not None else i
@@ -168,35 +186,39 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
         for line in cursor.description:
             info = field_info[line[0]]
             if DJANGO_VERSION < (3, 2, 0):
-                fields.append(FieldInfo(
-                    *line[:3],
-                    to_int(info.max_len) or line[3],
-                    to_int(info.num_prec) or line[4],
-                    to_int(info.num_scale) or line[5],
-                    line[6],
-                    info.column_default,
-                    info.extra,
-                    info.is_unsigned,
-                    line[0] in json_constraints
-                ))
+                fields.append(
+                    FieldInfo(
+                        *line[:3],
+                        to_int(info.max_len) or line[3],
+                        to_int(info.num_prec) or line[4],
+                        to_int(info.num_scale) or line[5],
+                        line[6],
+                        info.column_default,
+                        info.extra,
+                        info.is_unsigned,
+                        line[0] in json_constraints,
+                    )
+                )
             else:
-                fields.append(FieldInfo(
-                    *line[:3],
-                    to_int(info.max_len) or line[3],
-                    to_int(info.num_prec) or line[4],
-                    to_int(info.num_scale) or line[5],
-                    line[6],
-                    info.column_default,
-                    info.collation,
-                    info.extra,
-                    info.is_unsigned,
-                    line[0] in json_constraints,
-                ))
+                fields.append(
+                    FieldInfo(
+                        *line[:3],
+                        to_int(info.max_len) or line[3],
+                        to_int(info.num_prec) or line[4],
+                        to_int(info.num_scale) or line[5],
+                        line[6],
+                        info.column_default,
+                        info.collation,
+                        info.extra,
+                        info.is_unsigned,
+                        line[0] in json_constraints,
+                    )
+                )
         return fields
 
     def get_indexes(self, cursor, table_name):
-        cursor.execute("SHOW INDEX FROM {0}"
-                       "".format(self.connection.ops.quote_name(table_name)))
+        """Return indexes from table."""
+        cursor.execute(f"SHOW INDEX FROM {self.connection.ops.quote_name(table_name)}")
         # Do a two-pass search for indexes: on first pass check which indexes
         # are multicolumn, on second pass check which single-column indexes
         # are present.
@@ -210,13 +232,13 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
             if row[2] in multicol_indexes:
                 continue
             if row[4] not in indexes:
-                indexes[row[4]] = {'primary_key': False, 'unique': False}
+                indexes[row[4]] = {"primary_key": False, "unique": False}
             # It's possible to have the unique and PK constraints in
             # separate indexes.
-            if row[2] == 'PRIMARY':
-                indexes[row[4]]['primary_key'] = True
+            if row[2] == "PRIMARY":
+                indexes[row[4]]["primary_key"] = True
             if not row[1]:
-                indexes[row[4]]['unique'] = True
+                indexes[row[4]]["unique"] = True
         return indexes
 
     def get_primary_key_column(self, cursor, table_name):
@@ -224,15 +246,15 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
         Returns the name of the primary key column for the given table
         """
         for column in self.get_indexes(cursor, table_name).items():
-            if column[1]['primary_key']:
+            if column[1]["primary_key"]:
                 return column[0]
         return None
 
     def get_sequences(self, cursor, table_name, table_fields=()):
         for field_info in self.get_table_description(cursor, table_name):
-            if 'auto_increment' in field_info.extra:
+            if "auto_increment" in field_info.extra:
                 # MySQL allows only one auto-increment column per table.
-                return [{'table': table_name, 'column': field_info.name}]
+                return [{"table": table_name, "column": field_info.name}]
         return []
 
     def get_relations(self, cursor, table_name):
@@ -252,13 +274,16 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
         for all key columns in the given table.
         """
         key_columns = []
-        cursor.execute("""
+        cursor.execute(
+            """
             SELECT column_name, referenced_table_name, referenced_column_name
             FROM information_schema.key_column_usage
             WHERE table_name = %s
                 AND table_schema = DATABASE()
                 AND referenced_table_name IS NOT NULL
-                AND referenced_column_name IS NOT NULL""", [table_name])
+                AND referenced_column_name IS NOT NULL""",
+            [table_name],
+        )
         key_columns.extend(cursor.fetchall())
         return key_columns
 
@@ -268,14 +293,29 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
         storage engine if the table doesn't exist.
         """
         cursor.execute(
-            "SELECT engine "
-            "FROM information_schema.tables "
-            "WHERE table_name = %s", [table_name])
+            "SELECT engine FROM information_schema.tables WHERE table_name = %s",
+            [table_name],
+        )
         result = cursor.fetchone()
+        # pylint: disable=protected-access
         if not result:
             return self.connection.features._mysql_storage_engine
+        # pylint: enable=protected-access
         return result[0]
 
+    def _parse_constraint_columns(self, check_clause, columns):
+        check_columns = OrderedSet()
+        statement = sqlparse.parse(check_clause)[0]
+        tokens = (token for token in statement.flatten() if not token.is_whitespace)
+        for token in tokens:
+            if (
+                token.ttype == sqlparse.tokens.Name
+                and self.connection.ops.quote_name(token.value) == token.value
+                and token.value[1:-1] in columns
+            ):
+                check_columns.add(token.value[1:-1])
+        return check_columns
+
     def get_constraints(self, cursor, table_name):
         """
         Retrieve any constraints or keys (unique, pk, fk, check, index) across
@@ -296,16 +336,16 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
         for constraint, column, ref_table, ref_column in cursor.fetchall():
             if constraint not in constraints:
                 constraints[constraint] = {
-                    'columns': OrderedSet(),
-                    'primary_key': False,
-                    'unique': False,
-                    'index': False,
-                    'check': False,
-                    'foreign_key': (ref_table, ref_column) if ref_column else None,
+                    "columns": OrderedSet(),
+                    "primary_key": False,
+                    "unique": False,
+                    "index": False,
+                    "check": False,
+                    "foreign_key": (ref_table, ref_column) if ref_column else None,
                 }
                 if self.connection.features.supports_index_column_ordering:
-                    constraints[constraint]['orders'] = []
-            constraints[constraint]['columns'].add(column)
+                    constraints[constraint]["orders"] = []
+            constraints[constraint]["columns"].add(column)
         # Now get the constraint types
         type_query = """
             SELECT c.constraint_name, c.constraint_type
@@ -317,14 +357,16 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
         cursor.execute(type_query, [table_name])
         for constraint, kind in cursor.fetchall():
             if kind.lower() == "primary key":
-                constraints[constraint]['primary_key'] = True
-                constraints[constraint]['unique'] = True
+                constraints[constraint]["primary_key"] = True
+                constraints[constraint]["unique"] = True
             elif kind.lower() == "unique":
-                constraints[constraint]['unique'] = True
+                constraints[constraint]["unique"] = True
         # Add check constraints.
         if self.connection.features.can_introspect_check_constraints:
             unnamed_constraints_index = 0
-            columns = {info.name for info in self.get_table_description(cursor, table_name)}
+            columns = {
+                info.name for info in self.get_table_description(cursor, table_name)
+            }
             type_query = """
                 SELECT cc.constraint_name, cc.check_clause
                 FROM
@@ -339,42 +381,46 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
             """
             cursor.execute(type_query, [table_name])
             for constraint, check_clause in cursor.fetchall():
-                constraint_columns = self._parse_constraint_columns(check_clause, columns)
+                constraint_columns = self._parse_constraint_columns(
+                    check_clause, columns
+                )
                 # Ensure uniqueness of unnamed constraints. Unnamed unique
                 # and check columns constraints have the same name as
                 # a column.
                 if set(constraint_columns) == {constraint}:
                     unnamed_constraints_index += 1
-                    constraint = '__unnamed_constraint_%s__' % unnamed_constraints_index
+                    constraint = f"__unnamed_constraint_{unnamed_constraints_index}__"
                 constraints[constraint] = {
-                    'columns': constraint_columns,
-                    'primary_key': False,
-                    'unique': False,
-                    'index': False,
-                    'check': True,
-                    'foreign_key': None,
+                    "columns": constraint_columns,
+                    "primary_key": False,
+                    "unique": False,
+                    "index": False,
+                    "check": True,
+                    "foreign_key": None,
                 }
         # Now add in the indexes
-        cursor.execute("SHOW INDEX FROM %s" % self.connection.ops.quote_name(table_name))
-        for table, non_unique, index, colseq, column, order, type_ in [
+        cursor.execute(f"SHOW INDEX FROM {self.connection.ops.quote_name(table_name)}")
+        for _, _, index, _, column, order, type_ in [
             x[:6] + (x[10],) for x in cursor.fetchall()
         ]:
             if index not in constraints:
                 constraints[index] = {
-                    'columns': OrderedSet(),
-                    'primary_key': False,
-                    'unique': False,
-                    'check': False,
-                    'foreign_key': None,
+                    "columns": OrderedSet(),
+                    "primary_key": False,
+                    "unique": False,
+                    "check": False,
+                    "foreign_key": None,
                 }
                 if self.connection.features.supports_index_column_ordering:
-                    constraints[index]['orders'] = []
-            constraints[index]['index'] = True
-            constraints[index]['type'] = Index.suffix if type_ == 'BTREE' else type_.lower()
-            constraints[index]['columns'].add(column)
+                    constraints[index]["orders"] = []
+            constraints[index]["index"] = True
+            constraints[index]["type"] = (
+                Index.suffix if type_ == "BTREE" else type_.lower()
+            )
+            constraints[index]["columns"].add(column)
             if self.connection.features.supports_index_column_ordering:
-                constraints[index]['orders'].append('DESC' if order == 'D' else 'ASC')
+                constraints[index]["orders"].append("DESC" if order == "D" else "ASC")
         # Convert the sorted sets to lists
         for constraint in constraints.values():
-            constraint['columns'] = list(constraint['columns'])
+            constraint["columns"] = list(constraint["columns"])
         return constraints
