@@ -1,4 +1,4 @@
-# Copyright (c) 2009, 2020, Oracle and/or its affiliates.
+# Copyright (c) 2009, 2022, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -26,11 +26,10 @@
 # along with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 
-"""Python exceptions
-"""
+"""Python exceptions."""
 
-from . import utils
 from .locales import get_client_error
+from .utils import read_bytes, read_int
 
 # _CUSTOM_ERROR_EXCEPTIONS holds custom exceptions and is ued by the
 # function custom_error_exception. _ERROR_EXCEPTIONS (at bottom of module)
@@ -74,7 +73,7 @@ def custom_error_exception(error=None, exception=None):
 
     Returns a dictionary.
     """
-    global _CUSTOM_ERROR_EXCEPTIONS  # pylint: disable=W0603
+    global _CUSTOM_ERROR_EXCEPTIONS  # pylint: disable=global-statement
 
     if isinstance(error, dict) and not error:
         _CUSTOM_ERROR_EXCEPTIONS = {}
@@ -84,24 +83,24 @@ def custom_error_exception(error=None, exception=None):
         return _CUSTOM_ERROR_EXCEPTIONS
 
     if not isinstance(error, (int, dict)):
-        raise ValueError(
-            "The error argument should be either an integer or dictionary")
+        raise ValueError("The error argument should be either an integer or dictionary")
 
     if isinstance(error, int):
         error = {error: exception}
 
     for errno, _exception in error.items():
         if not isinstance(errno, int):
-            raise ValueError("error number should be an integer")
+            raise ValueError("Error number should be an integer")
         try:
             if not issubclass(_exception, Exception):
                 raise TypeError
-        except TypeError:
-            raise ValueError("exception should be subclass of Exception")
+        except TypeError as err:
+            raise ValueError("Exception should be subclass of Exception") from err
         _CUSTOM_ERROR_EXCEPTIONS[errno] = _exception
 
     return _CUSTOM_ERROR_EXCEPTIONS
 
+
 def get_mysql_exception(errno, msg=None, sqlstate=None):
     """Get the exception matching the MySQL error
 
@@ -114,15 +113,13 @@ def get_mysql_exception(errno, msg=None, sqlstate=None):
     Returns an Exception
     """
     try:
-        return _CUSTOM_ERROR_EXCEPTIONS[errno](
-            msg=msg, errno=errno, sqlstate=sqlstate)
+        return _CUSTOM_ERROR_EXCEPTIONS[errno](msg=msg, errno=errno, sqlstate=sqlstate)
     except KeyError:
         # Error was not mapped to particular exception
         pass
 
     try:
-        return _ERROR_EXCEPTIONS[errno](
-            msg=msg, errno=errno, sqlstate=sqlstate)
+        return _ERROR_EXCEPTIONS[errno](msg=msg, errno=errno, sqlstate=sqlstate)
     except KeyError:
         # Error was not mapped to particular exception
         pass
@@ -132,11 +129,13 @@ def get_mysql_exception(errno, msg=None, sqlstate=None):
 
     try:
         return _SQLSTATE_CLASS_EXCEPTION[sqlstate[0:2]](
-            msg=msg, errno=errno, sqlstate=sqlstate)
+            msg=msg, errno=errno, sqlstate=sqlstate
+        )
     except KeyError:
         # Return default InterfaceError
         return DatabaseError(msg=msg, errno=errno, sqlstate=sqlstate)
 
+
 def get_exception(packet):
     """Returns an exception object based on the MySQL error
 
@@ -151,32 +150,33 @@ def get_exception(packet):
         if packet[4] != 255:
             raise ValueError("Packet is not an error packet")
     except IndexError as err:
-        return InterfaceError("Failed getting Error information (%r)" % err)
+        return InterfaceError(f"Failed getting Error information ({err})")
 
     sqlstate = None
     try:
         packet = packet[5:]
-        (packet, errno) = utils.read_int(packet, 2)
+        packet, errno = read_int(packet, 2)
         if packet[0] != 35:
             # Error without SQLState
             if isinstance(packet, (bytes, bytearray)):
-                errmsg = packet.decode('utf8')
+                errmsg = packet.decode("utf8")
             else:
                 errmsg = packet
         else:
-            (packet, sqlstate) = utils.read_bytes(packet[1:], 5)
-            sqlstate = sqlstate.decode('utf8')
-            errmsg = packet.decode('utf8')
-    except Exception as err:  # pylint: disable=W0703
-        return InterfaceError("Failed getting Error information (%r)" % err)
+            packet, sqlstate = read_bytes(packet[1:], 5)
+            sqlstate = sqlstate.decode("utf8")
+            errmsg = packet.decode("utf8")
+    except (IndexError, UnicodeError) as err:
+        return InterfaceError(f"Failed getting Error information ({err})")
     else:
         return get_mysql_exception(errno, errmsg, sqlstate)
 
 
 class Error(Exception):
     """Exception that is base class for all other error exceptions"""
+
     def __init__(self, msg=None, errno=None, values=None, sqlstate=None):
-        super(Error, self).__init__()
+        super().__init__()
         self.msg = msg
         self._full_msg = self.msg
         self.errno = errno or -1
@@ -188,20 +188,17 @@ class Error(Exception):
                 try:
                     self.msg = self.msg % values
                 except TypeError as err:
-                    self.msg = "{0} (Warning: {1})".format(self.msg, str(err))
+                    self.msg = f"{self.msg} (Warning: {err})"
         elif not self.msg:
-            self._full_msg = self.msg = 'Unknown error'
+            self._full_msg = self.msg = "Unknown error"
 
         if self.msg and self.errno != -1:
-            fields = {
-                'errno': self.errno,
-                'msg': self.msg
-            }
+            fields = {"errno": self.errno, "msg": self.msg}
             if self.sqlstate:
-                fmt = '{errno} ({state}): {msg}'
-                fields['state'] = self.sqlstate
+                fmt = "{errno} ({state}): {msg}"
+                fields["state"] = self.sqlstate
             else:
-                fmt = '{errno}: {msg}'
+                fmt = "{errno}: {msg}"
             self._full_msg = fmt.format(**fields)
 
         self.args = (self.errno, self._full_msg, self.sqlstate)
@@ -210,88 +207,78 @@ class Error(Exception):
         return self._full_msg
 
 
-class Warning(Exception):  # pylint: disable=W0622
+class Warning(Exception):  # pylint: disable=redefined-builtin
     """Exception for important warnings"""
-    pass
 
 
 class InterfaceError(Error):
     """Exception for errors related to the interface"""
-    pass
 
 
 class DatabaseError(Error):
     """Exception for errors related to the database"""
-    pass
 
 
 class InternalError(DatabaseError):
     """Exception for errors internal database errors"""
-    pass
 
 
 class OperationalError(DatabaseError):
     """Exception for errors related to the database's operation"""
-    pass
 
 
 class ProgrammingError(DatabaseError):
     """Exception for errors programming errors"""
-    pass
 
 
 class IntegrityError(DatabaseError):
     """Exception for errors regarding relational integrity"""
-    pass
 
 
 class DataError(DatabaseError):
     """Exception for errors reporting problems with processed data"""
-    pass
 
 
 class NotSupportedError(DatabaseError):
     """Exception for errors when an unsupported database feature was used"""
-    pass
 
 
 class PoolError(Error):
     """Exception for errors relating to connection pooling"""
-    pass
 
 
 _SQLSTATE_CLASS_EXCEPTION = {
-    '02': DataError,  # no data
-    '07': DatabaseError,  # dynamic SQL error
-    '08': OperationalError,  # connection exception
-    '0A': NotSupportedError,  # feature not supported
-    '21': DataError,  # cardinality violation
-    '22': DataError,  # data exception
-    '23': IntegrityError,  # integrity constraint violation
-    '24': ProgrammingError,  # invalid cursor state
-    '25': ProgrammingError,  # invalid transaction state
-    '26': ProgrammingError,  # invalid SQL statement name
-    '27': ProgrammingError,  # triggered data change violation
-    '28': ProgrammingError,  # invalid authorization specification
-    '2A': ProgrammingError,  # direct SQL syntax error or access rule violation
-    '2B': DatabaseError,  # dependent privilege descriptors still exist
-    '2C': ProgrammingError,  # invalid character set name
-    '2D': DatabaseError,  # invalid transaction termination
-    '2E': DatabaseError,  # invalid connection name
-    '33': DatabaseError,  # invalid SQL descriptor name
-    '34': ProgrammingError,  # invalid cursor name
-    '35': ProgrammingError,  # invalid condition number
-    '37': ProgrammingError,  # dynamic SQL syntax error or access rule violation
-    '3C': ProgrammingError,  # ambiguous cursor name
-    '3D': ProgrammingError,  # invalid catalog name
-    '3F': ProgrammingError,  # invalid schema name
-    '40': InternalError,  # transaction rollback
-    '42': ProgrammingError,  # syntax error or access rule violation
-    '44': InternalError,   # with check option violation
-    'HZ': OperationalError,  # remote database access
-    'XA': IntegrityError,
-    '0K': OperationalError,
-    'HY': DatabaseError,  # default when no SQLState provided by MySQL server
+    "02": DataError,  # no data
+    "07": DatabaseError,  # dynamic SQL error
+    "08": OperationalError,  # connection exception
+    "0A": NotSupportedError,  # feature not supported
+    "21": DataError,  # cardinality violation
+    "22": DataError,  # data exception
+    "23": IntegrityError,  # integrity constraint violation
+    "24": ProgrammingError,  # invalid cursor state
+    "25": ProgrammingError,  # invalid transaction state
+    "26": ProgrammingError,  # invalid SQL statement name
+    "27": ProgrammingError,  # triggered data change violation
+    "28": ProgrammingError,  # invalid authorization specification
+    "2A": ProgrammingError,  # direct SQL syntax error or access rule violation
+    "2B": DatabaseError,  # dependent privilege descriptors still exist
+    "2C": ProgrammingError,  # invalid character set name
+    "2D": DatabaseError,  # invalid transaction termination
+    "2E": DatabaseError,  # invalid connection name
+    "33": DatabaseError,  # invalid SQL descriptor name
+    "34": ProgrammingError,  # invalid cursor name
+    "35": ProgrammingError,  # invalid condition number
+    "37": ProgrammingError,  # dynamic SQL syntax error or access rule violation
+    "3C": ProgrammingError,  # ambiguous cursor name
+    "3D": ProgrammingError,  # invalid catalog name
+    "3F": ProgrammingError,  # invalid schema name
+    "40": InternalError,  # transaction rollback
+    "42": ProgrammingError,  # syntax error or access rule violation
+    "44": InternalError,  # with check option violation
+    "HZ": OperationalError,  # remote database access
+    "XA": IntegrityError,
+    "0K": OperationalError,
+    "HY": DatabaseError,  # default when no SQLState provided by MySQL server
 }
 
 _ERROR_EXCEPTIONS = {
