@@ -1,4 +1,4 @@
-# Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2016, 2022, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -30,21 +30,21 @@
 
 import json
 
-from .compat import STRING_TYPES
 from .errors import ProgrammingError
 
 
 class ExprJSONEncoder(json.JSONEncoder):
     """A :class:`json.JSONEncoder` subclass, which enables encoding of
     :class:`mysqlx.ExprParser` objects."""
-    def default(self, o):  # pylint: disable=E0202
+
+    def default(self, o):
         if hasattr(o, "expr"):
-            return "{0}".format(o)
+            return f"{o}"
         # Let the base class default method raise the TypeError
         return json.JSONEncoder.default(self, o)
 
 
-class DbDoc(object):
+class DbDoc:
     """Represents a generic document in JSON format.
 
     Args:
@@ -53,13 +53,14 @@ class DbDoc(object):
     Raises:
         ValueError: If ``value`` type is not a basestring or dict.
     """
+
     def __init__(self, value):
         if isinstance(value, dict):
             self.__dict__ = value
-        elif isinstance(value, STRING_TYPES):
+        elif isinstance(value, str):
             self.__dict__ = json.loads(value)
         else:
-            raise ValueError("Unable to handle type: {0}".format(type(value)))
+            raise ValueError(f"Unable to handle type: {type(value)}")
 
     def __str__(self):
         return self.as_str()
@@ -75,6 +76,9 @@ class DbDoc(object):
     def __getitem__(self, index):
         return self.__dict__[index]
 
+    def __contains__(self, item):
+        return item in self.__dict__
+
     def copy(self, doc_id=None):
         """Returns a new copy of a :class:`mysqlx.DbDoc` object containing the
         `doc_id` provided. If `doc_id` is not provided, it will be removed from
