@@ -1,4 +1,4 @@
-# Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2017, 2022, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -25,16 +25,21 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+
 """This module contains helper functions."""
 
+import binascii
+import decimal
 import functools
 import inspect
 import warnings
 
-from .compat import NUMERIC_TYPES
 from .constants import TLS_CIPHER_SUITES, TLS_VERSIONS
 from .errors import InterfaceError
 
+BYTE_TYPES = (bytearray, bytes)
+NUMERIC_TYPES = (int, float, decimal.Decimal)
+
 
 def encode_to_bytes(value, encoding="utf-8"):
     """Returns an encoded version of the string as a bytes object.
@@ -85,27 +90,29 @@ def escape(*args):
     Returns:
         str: The value if not a string, or the escaped string.
     """
+
     def _escape(value):
         """Escapes special characters."""
         if value is None:
             return value
-        elif isinstance(value, NUMERIC_TYPES):
+        if isinstance(value, NUMERIC_TYPES):
             return value
         if isinstance(value, (bytes, bytearray)):
-            value = value.replace(b'\\', b'\\\\')
-            value = value.replace(b'\n', b'\\n')
-            value = value.replace(b'\r', b'\\r')
-            value = value.replace(b'\047', b'\134\047')  # single quotes
-            value = value.replace(b'\042', b'\134\042')  # double quotes
-            value = value.replace(b'\032', b'\134\032')  # for Win32
+            value = value.replace(b"\\", b"\\\\")
+            value = value.replace(b"\n", b"\\n")
+            value = value.replace(b"\r", b"\\r")
+            value = value.replace(b"\047", b"\134\047")  # single quotes
+            value = value.replace(b"\042", b"\134\042")  # double quotes
+            value = value.replace(b"\032", b"\134\032")  # for Win32
         else:
-            value = value.replace('\\', '\\\\')
-            value = value.replace('\n', '\\n')
-            value = value.replace('\r', '\\r')
-            value = value.replace('\047', '\134\047')  # single quotes
-            value = value.replace('\042', '\134\042')  # double quotes
-            value = value.replace('\032', '\134\032')  # for Win32
+            value = value.replace("\\", "\\\\")
+            value = value.replace("\n", "\\n")
+            value = value.replace("\r", "\\r")
+            value = value.replace("\047", "\134\047")  # single quotes
+            value = value.replace("\042", "\134\042")  # double quotes
+            value = value.replace("\032", "\134\032")  # for Win32
         return value
+
     if len(args) > 1:
         return [_escape(arg) for arg in args]
     return _escape(args[0])
@@ -123,8 +130,10 @@ def quote_identifier(identifier, sql_mode=""):
         str: Returns string with the identifier quoted with backticks.
     """
     if sql_mode == "ANSI_QUOTES":
-        return '"{0}"'.format(identifier.replace('"', '""'))
-    return "`{0}`".format(identifier.replace("`", "``"))
+        quoted = identifier.replace('"', '""')
+        return f'"{quoted}"'
+    quoted = identifier.replace("`", "``")
+    return f"`{quoted}`"
 
 
 def deprecated(version=None, reason=None):
@@ -144,8 +153,10 @@ def deprecated(version=None, reason=None):
        def deprecated_function(x, y):
            return x + y
     """
+
     def decorate(func):
         """Decorate function."""
+
         @functools.wraps(func)
         def wrapper(*args, **kwargs):
             """Wrapper function.
@@ -154,20 +165,25 @@ def deprecated(version=None, reason=None):
                 *args: Variable length argument list.
                 **kwargs: Arbitrary keyword arguments.
             """
-            message = ["'{}' is deprecated".format(func.__name__)]
+            message = [f"'{func.__name__}' is deprecated"]
             if version:
-                message.append(" since version {}".format(version))
+                message.append(f" since version {version}")
             if reason:
-                message.append(". {}".format(reason))
+                message.append(f". {reason}")
             frame = inspect.currentframe().f_back
-            warnings.warn_explicit("".join(message),
-                                   category=DeprecationWarning,
-                                   filename=inspect.getfile(frame.f_code),
-                                   lineno=frame.f_lineno)
+            warnings.warn_explicit(
+                "".join(message),
+                category=DeprecationWarning,
+                filename=inspect.getfile(frame.f_code),
+                lineno=frame.f_lineno,
+            )
             return func(*args, **kwargs)
+
         return wrapper
+
     return decorate
 
+
 def iani_to_openssl_cs_name(tls_version, cipher_suites_names):
     """Translates a cipher suites names list; from IANI names to OpenSSL names.
 
@@ -177,7 +193,7 @@ def iani_to_openssl_cs_name(tls_version, cipher_suites_names):
     """
     translated_names = []
 
-    cipher_suites = {}#TLS_CIPHER_SUITES[TLS_version]
+    cipher_suites = {}  # TLS_CIPHER_SUITES[TLS_version]
 
     # Find the previews TLS versions of the given on TLS_version
     for index in range(TLS_VERSIONS.index(tls_version) + 1):
@@ -189,6 +205,19 @@ def iani_to_openssl_cs_name(tls_version, cipher_suites_names):
         elif name in cipher_suites:
             translated_names.append(cipher_suites[name])
         else:
-            raise InterfaceError("The '{}' in cipher suites is not a valid "
-                                 "cipher suite".format(name))
+            raise InterfaceError(
+                f"The '{name}' in cipher suites is not a valid cipher suite"
+            )
     return translated_names
+
+
+def hexlify(data):
+    """Return the hexadecimal representation of the binary data.
+
+    Args:
+        data (str): The binary data.
+
+    Returns:
+        bytes: The hexadecimal representation of data.
+    """
+    return binascii.hexlify(data).decode("utf-8")
